@@ -1,11 +1,11 @@
 /*
 	Preamp.cc
 	
-	Copyright 2003-4 Tim Goetze <tim@quitte.de>
+	Copyright 2003-7 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
-	loosely 12AX7-based tube preamp model, 8x oversampling.
+	Loosely 12AX7-based tube preamp model with 8x oversampling.
 */
 /*
 	This program is free software; you can redistribute it and/or
@@ -31,11 +31,11 @@
 #include "Descriptor.h"
 
 void
-PreampIII::init (double _fs)
+PreampIII::init()
 {
-	this->AmpStub::init (_fs);
+	this->AmpStub::init();
 
-	DSP::RBJ::LoShelve (200 / (_fs), .2, -6, boost.a, boost.b);
+	DSP::RBJ::LoShelve (200 / fs, .2, -6, filter.a, filter.b);
 }
 
 template <sample_func_t F, int OVERSAMPLE>
@@ -43,21 +43,21 @@ void
 PreampIII::one_cycle (int frames)
 {
 	d_sample * s = ports[0];
-	d_sample gain = *ports[1];
-	d_sample temp = *ports[2] * tube.scale;
+	d_sample gain = getport(1);
+	d_sample temp = getport(2) * tube.scale;
 	d_sample * d = ports[3];
 	*ports[4] = OVERSAMPLE;
 
 	double g = current.g;
 
-	current.g = max (gain < 1 ? gain : DSP::exp2_approx (gain - 1), .000001);
+	current.g = max (gain < 1 ? gain : exp2 (gain - 1), .000001);
 	/* correction for attenuated first transfer */	
 	current.g *= tube.scale / fabs (tube.transfer (temp)); 
 
 	if (g == 0) g = current.g;
 	
 	/* recursive fade to prevent zipper noise from the 'gain' knob */
-	double one_over_n = 1. / frames;
+	double one_over_n = frames > 0 ? 1. / frames : 1;
 	double gf = pow (current.g / g, one_over_n);
 
 	for (int i = 0; i < frames; ++i)
@@ -65,7 +65,7 @@ PreampIII::one_cycle (int frames)
 		register d_sample a = s[i] + normal;
 
 		a = g * tube.transfer (a * temp);
-		a = boost.process (a);
+		a = filter.process (a);
 		a = down.process (tube.transfer_clip (up.upsample (a)));
 
 		for (int o = 1; o < OVERSAMPLE; ++o)
@@ -77,7 +77,6 @@ PreampIII::one_cycle (int frames)
 	}
 
 	current.g = g;
-	normal = -normal;
 }
 
 /* //////////////////////////////////////////////////////////////////////// */
@@ -108,16 +107,16 @@ PreampIII::port_info [] =
 	}
 };
 
-void
+template <> void
 Descriptor<PreampIII>::setup()
 {
 	UniqueID = 1776;
 	Label = "PreampIII";
 	Properties = HARD_RT;
 
-	Name = "Tube preamp emulation";
+	Name = CAPS "PreampIII - Tube preamp emulation";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2002-4";
+	Copyright = "GPL, 2002-7";
 
 	/* fill port info and vtable */
 	autogen();
@@ -126,11 +125,10 @@ Descriptor<PreampIII>::setup()
 /* //////////////////////////////////////////////////////////////////////// */
 
 void
-PreampIV::init (double _fs)
+PreampIV::init()
 {
-	this->AmpStub::init (_fs);
-
-	tone.init (_fs);
+	this->AmpStub::init();
+	tone.init (fs);
 }
 
 void
@@ -145,11 +143,11 @@ template <sample_func_t F, int OVERSAMPLE>
 void
 PreampIV::one_cycle (int frames)
 {
-	double one_over_n = 1. / frames;
+	double one_over_n = frames > 0 ? 1. / frames : 1;
 
 	d_sample * s = ports[0];
-	d_sample gain = *ports[1];
-	d_sample temp = *ports[2] * tube.scale;
+	d_sample gain = getport(1);
+	d_sample temp = getport(2) * tube.scale;
 
 	tone.start_cycle (ports + 3, one_over_n);
 
@@ -158,7 +156,7 @@ PreampIV::one_cycle (int frames)
 
 	double g = current.g;
 
-	current.g = max (gain < 1 ? gain : DSP::exp2_approx (gain - 1), .000001);
+	current.g = max (gain < 1 ? gain : exp2 (gain - 1), .000001);
 	current.g *= tube.scale / fabs (tube.transfer (temp)); 
 
 	if (g == 0) g = current.g;
@@ -183,7 +181,6 @@ PreampIV::one_cycle (int frames)
 	}
 
 	current.g = g;
-	normal = -normal;
 }
 
 /* //////////////////////////////////////////////////////////////////////// */
@@ -230,16 +227,16 @@ PreampIV::port_info [] =
 	}
 };
 
-void
+template <> void
 Descriptor<PreampIV>::setup()
 {
 	UniqueID = 1777;
 	Label = "PreampIV";
 	Properties = HARD_RT;
 
-	Name = "Tube preamp emulation + tone controls";
+	Name = CAPS "PreampIV - Tube preamp emulation + tone controls";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2002-4";
+	Copyright = "GPL, 2002-7";
 
 	/* fill port info and vtable */
 	autogen();
