@@ -1,7 +1,7 @@
 /*
 	basics.h
 	
-	Copyright 2004-5 Tim Goetze <tim@quitte.de>
+	Copyright 2004-9 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
@@ -32,7 +32,7 @@
 #define _GNU_SOURCE 1
 #define _USE_GNU 1
 
-/* gcc protects a lot of standard math calls. */
+/* unlocking some standard math calls. */
 #define __USE_ISOC99 1
 #define __USE_ISOC9X 1
 #define _ISOC99_SOURCE 1
@@ -48,9 +48,9 @@
 
 #include "ladspa.h"
 
+/* reducing LADSPA_DEFINES_WITH_LOTS_OF_CHARACTERS_REALLY verbosity */
 #define BOUNDED (LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE)
 #define INTEGER LADSPA_HINT_INTEGER
-#define FS LADSPA_HINT_SAMPLE_RATE
 #define LOG LADSPA_HINT_LOGARITHMIC
 #define TOGGLE LADSPA_HINT_TOGGLED
 
@@ -74,6 +74,8 @@
 #define TEN_TO_THE_SIXTH 1000000
 
 #define MIN_GAIN .000001 /* -120 dB */
+
+/* smallest non-denormal 32 bit IEEE float is 1.1810-38 */
 #define NOISE_FLOOR .00000000000005 /* -266 dB */
 
 typedef __int8_t			int8;
@@ -86,7 +88,7 @@ typedef __int64_t			int64;
 typedef __uint64_t		uint64;
 
 typedef struct {
-	char * name;
+	const char * name;
 	LADSPA_PortDescriptor descriptor;
 	LADSPA_PortRangeHint range;
 } PortInfo;
@@ -125,11 +127,73 @@ X max (X x, Y y)
 }
 
 #endif /* ! max */
-	
+
+template <class T>
+T clamp (T value, T lower, T upper)
+{
+	if (value < lower) return lower;
+	if (value > upper) return upper;
+	return value;
+}
+
 static inline float
 frandom()
 {
 	return (float) random() / (float) RAND_MAX;
 }
 
+/* NB: also true if 0  */
+inline bool 
+is_denormal (float & f)
+{
+	int32 i = *((int32 *) &f);
+	return ((i & 0x7f800000) == 0);
+}
+
+/* todo: not sure if this double version is correct, actually ... */
+inline bool 
+is_denormal (double & f)
+{
+	int64 i = *((int64 *) &f);
+	return ((i & 0x7fe0000000000000ll) == 0);
+}
+
+#ifdef __i386__
+	#define TRAP asm ("int $3;")
+#else
+	#define TRAP
+#endif
+
+/* //////////////////////////////////////////////////////////////////////// */
+
+#define CAPS "C* "
+
+class Plugin {
+	public:
+		double fs; /* sample rate */
+		double adding_gain; /* for run_adding() */
+
+		int first_run; /* 1st block after activate(), do no parameter smoothing */
+		d_sample normal; /* renormal constant */
+
+		d_sample ** ports;
+		LADSPA_PortRangeHint * ranges; /* for getport() below */
+
+	public:
+		/* get port value, mapping inf or nan to 0 */
+		inline d_sample getport_unclamped (int i)
+			{
+				d_sample v = *ports[i];
+				return (isinf (v) || isnan(v)) ? 0 : v;
+			}
+
+		/* get port value and clamp to port range */
+		inline d_sample getport (int i)
+			{
+				LADSPA_PortRangeHint & r = ranges[i];
+				d_sample v = getport_unclamped (i);
+				return clamp (v, r.LowerBound, r.UpperBound);
+			}
+};
+
 #endif /* _BASICS_H_ */
