@@ -2,6 +2,7 @@
 import os
 
 CFLAGS = []
+OSX_LDFLAGS = "-bundle -undefined suppress -flat_namespace"
 
 def we_have_sse():
 	try: return 'sse' in open ('/proc/cpuinfo').read().split()
@@ -10,10 +11,17 @@ def we_have_ssse3():
 	try: return 'ssse3' in open ('/proc/cpuinfo').read().split()
 	except: return 0
 
+def we_think_so_different_dude():
+	try: return 'Darwin' == os.popen ('uname -s').read().strip()
+	except: return 0
+
 def store():
 	f = open ('defines.make', 'w')
 	print >> f, "_CFLAGS=" + ' '.join (CFLAGS)
-
+	if we_think_so_different_dude():
+		print >> f, "_LDFLAGS=", OSX_LDFLAGS
+		print >> f, "STRIP = echo"
+	
 if __name__ == '__main__':
 	if we_have_sse():
 		CFLAGS += ('-msse', '-mfpmath=sse')
